/*
 * Access the keypad to modify the backlight on the Letux 400.
 *
 * Fn+Down/Up adjusts the brightness.
 * Zzz+Down/Up disables/enables the backlight.
 *
 * Copyright (C) 2018, 2020 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/devices/input-keypad-client.h>
#include <l4/devices/keypad-loader.h>

#include <l4/re/env.h>

#include <l4/re/event_enums.h>
#include <l4/util/util.h>

#include "activation_client.h"
#include "backlight_client.h"

/* Backlight level. */

enum Letux400_backlight_levels
{
  Letux400_backlight_min_level = 0,
  Letux400_backlight_max_level = 300,
  Letux400_backlight_step = 25,
};

static int backlight_level = 250;

/* Key state. */

static int modifier_set = 0;
static int enable_modifier_set = 0;

/* Backlight references. */

Activation *activation_device;
Backlight *backlight_device;



/* Input event handler. */

static void handler(Input_event event, void *priv)
{
  (void) priv;

  /* Track the state of the modifier keys. */

  if (event.code == L4RE_KEY_FN)
    modifier_set = event.value;
  else if (event.code == L4RE_KEY_SLEEP)
    enable_modifier_set = event.value;

  if (!event.value || !(modifier_set || enable_modifier_set))
    return;

  /* Upon keypress events, test controls and update the backlight. */

  switch (event.code)
  {
    case L4RE_KEY_DOWN:
      if (enable_modifier_set)
      {
        activation_device->disable();
        return;
      }
      else if (backlight_level < Letux400_backlight_min_level + Letux400_backlight_step)
        backlight_level = Letux400_backlight_min_level;
      else
        backlight_level -= Letux400_backlight_step;
      break;

    case L4RE_KEY_UP:
      if (enable_modifier_set)
      {
        activation_device->enable();
        return;
      }
      else if (backlight_level > Letux400_backlight_max_level - Letux400_backlight_step)
        backlight_level = Letux400_backlight_max_level;
      else
        backlight_level += Letux400_backlight_step;
      break;

    default: return;
  }

  /* Use the backlight device to update the backlight level. */

  backlight_device->set_brightness(backlight_level);
}



int main(void)
{
  /* Load the keypad library using details from the configured library. */

  Keypad *keypad = (Keypad *) load_keypad();
  Input_keypad_client client(keypad);

  /* Obtain a reference to the backlight device. */

  l4_cap_idx_t backlight = l4re_env_get_cap("backlight");
  if (!l4_is_valid_cap(backlight)) return 1;

  /* NOTE: The component framework should permit the use of a compound client
           object. */

  client_Activation activation_obj(backlight);
  activation_device = &activation_obj;

  client_Backlight backlight_obj(backlight);
  backlight_device = &backlight_obj;

  client.attach(handler, 0);

  l4_sleep_forever();

  return 0;
}
